package com.lizlam.hayward_gems;

import android.app.Dialog;
import android.content.Intent;
import android.content.IntentSender;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesUtil;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.common.api.PendingResult;
import com.google.android.gms.common.api.ResultCallback;
import com.google.android.gms.location.places.Place;
import com.google.android.gms.location.places.PlaceBuffer;
import com.google.android.gms.location.places.Places;
import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;


public class MainActivity extends ActionBarActivity implements GoogleApiClient.ConnectionCallbacks, GoogleApiClient.OnConnectionFailedListener {

    private static final int GPS_ERRORDIALOG_REQUEST = 9001;

    GoogleMap mMap;
    Marker marker;
    GoogleApiClient mGoogleApiClient;
    TextView infoTextView;
    TextView websiteTextView;
    String locationName;
    String address;
    LatLng latlng;

    int zoom = 17;
    String meetUpDesc;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mGoogleApiClient = new GoogleApiClient.Builder(this)
                .addApi(Places.GEO_DATA_API)
                .addApi(Places.PLACE_DETECTION_API)
                .addConnectionCallbacks(this)
                .addOnConnectionFailedListener(this)
                .build();

        if (servicesOK()) {
            setContentView(R.layout.activity_main);
            if (initMap()) {
                gotoLocation(IMapConstants.HAYWARD_LAT, IMapConstants.HAYWARD_LONG, 15, false);
            } else {
                Toast.makeText(this, "No map!", Toast.LENGTH_SHORT).show();
            }
        } else {
            setContentView(R.layout.activity_main);
        }

        infoTextView = (TextView) findViewById(R.id.textView);
        websiteTextView = (TextView) findViewById(R.id.textView2);
    }

    @Override
    protected  void onStart() {
        super.onStart();
        mGoogleApiClient.connect();
    }

    @Override
    protected void onStop() {
        super.onStop();
        mGoogleApiClient.disconnect();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    public void setMap(String locName, String placeid, double lat, double lng) {
        clearAddress();
        latlng = gotoLocation(lat, lng, zoom);
        locationName = locName;
        getPlaceInfo(placeid);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();
        //noinspection SimplifiableIfStatement
        if (id == R.id.goto_tea_garden) {

            setMap(getString(R.string.tea_garden), IMapConstants.TEA_GARDEN_PLACE_ID, IMapConstants.TEA_GARDEN_LAT,
                    IMapConstants.TEA_GARDEN_LONG);
        } else if (id == R.id.goto_bills) {

            setMap(getString(R.string.bills), IMapConstants.BILLS_PLACE_ID, IMapConstants.BILLS_LAT,
                    IMapConstants.BILS_LONG);

        } else if (id == R.id.goto_bistro) {

            setMap(getString(R.string.bistro), IMapConstants.BISTRO_PLACE_ID, IMapConstants.BISTRO_LAT,
                    IMapConstants.BISTRO_LONG);

        } else if (id == R.id.goto_grid) {
            locationName = getString(R.string.grid);
            address = IMapConstants.OFF_THE_GRID_ADDRESS;
            latlng = gotoLocation(IMapConstants.OFF_THE_GRID_LAT, IMapConstants.OFF_THE_GRID_LONG, zoom);
            addInfo(IMapConstants.OFF_THE_GRID_HOURS, IMapConstants.OFF_THE_GRID_URL);
            setMarker(latlng, locationName, address);

        } else if (id == R.id.goto_vals) {

            setMap(getString(R.string.vals), IMapConstants.VALS_PLACE_ID, IMapConstants.VALS_LAT,
                    IMapConstants.VALS_LONG);

        } else if (id == R.id.goto_meetup) {
            MyTask mytask = new MyTask();
            mytask.execute();

        } else if (id == R.id.gps_license) {
            Intent intent = new Intent(this, GPSLicenseActivity.class);
            startActivity(intent);
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onConnected(Bundle bundle) {
        Log.d("MainActivity", "onConnected");
    }

    @Override
    public void onConnectionSuspended(int i) {
        Log.d("MainActivity", "onConnectionSuspended");
    }

    @Override
    public void onConnectionFailed(ConnectionResult connectionResult) {
        try {
            connectionResult.startResolutionForResult(this, 100);
        } catch (IntentSender.SendIntentException e) {
            e.printStackTrace();
        }
        Log.d("MainActivity", "onConnectionFailed");
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putString("infoviewText", infoTextView.getText().toString());
        if (websiteTextView != null) {
            outState.putString("websiteText", websiteTextView.getText().toString());
        }
        if (latlng != null) {
            outState.putDouble("lat", latlng.latitude);
            outState.putDouble("lng", latlng.longitude);
        }
        if ((locationName) != null && (address != null)) {
            outState.putString("locationName", locationName);
            outState.putString("address", address);
        }
    }

    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        if (savedInstanceState.get("locationName") != null) {
            locationName = savedInstanceState.get("locationName").toString();
            address = savedInstanceState.get("address").toString();
            latlng = new LatLng((double) savedInstanceState.get("lat"), (double) savedInstanceState.get("lng"));
            gotoLocation(latlng.latitude, latlng.longitude, zoom, false);
            addInfo(savedInstanceState.get("infoviewText").toString(), savedInstanceState.get("websiteText").toString());
            setMarker(latlng, locationName, address);
        }
    }

    public boolean servicesOK() {
        int isAvailable = GooglePlayServicesUtil.isGooglePlayServicesAvailable(this);
        if (isAvailable == ConnectionResult.SUCCESS) {
            return true;
        } else if (GooglePlayServicesUtil.isUserRecoverableError(isAvailable)) {
            Dialog dialog = GooglePlayServicesUtil.getErrorDialog(isAvailable, this, GPS_ERRORDIALOG_REQUEST);
            dialog.show();
        } else {
            Toast.makeText(this, "Can't connect to Google Play services", Toast.LENGTH_SHORT).show();
        }
        return false;
    }

    private boolean initMap() {
        if (mMap == null) {
            SupportMapFragment mapFragment =
                    (SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map);
            mMap = mapFragment.getMap();
        }
        return (mMap != null);
    }

    private LatLng gotoLocation(double lat, double lng, int zoom) {
        return gotoLocation(lat, lng, zoom, true);
    }

    private LatLng gotoLocation(double lat, double lng, float zoom, boolean animate) {
        latlng = new LatLng(lat, lng);
        CameraUpdate cameraUpdate = CameraUpdateFactory.newLatLngZoom(latlng, zoom);
        if (animate == false) {
            mMap.moveCamera(cameraUpdate);
        } else {
            mMap.animateCamera(cameraUpdate);
        }
        return latlng;
    }

    private void setMarker(LatLng pos, String title, String info) {
        if (marker != null) {
            removeMarker();
        }
        MarkerOptions options = new MarkerOptions()
                .title(title)
                .position(pos)
                .snippet(info)
                .icon(BitmapDescriptorFactory.defaultMarker(
                        BitmapDescriptorFactory.HUE_AZURE));

        marker = mMap.addMarker(options);
        marker.showInfoWindow();
    }

    private void removeMarker() {
        if (marker != null) {
            marker.remove();
            marker = null;
        }
    }

    private void clearAddress() {
        address = "";
    }

    private void addInfo(String info, String website) {
        infoTextView.setText(info);
        websiteTextView.setText(website);
    }

    // Find place id here:
    // https://developers.google.com/places/place-id#find-id
    private void getPlaceInfo(String placeid) {
        PendingResult<PlaceBuffer> placeResult = Places.GeoDataApi.getPlaceById(mGoogleApiClient, placeid);

        placeResult.setResultCallback(new ResultCallback<PlaceBuffer>() {
            @Override
            public void onResult(PlaceBuffer places) {
                infoTextView.setText("");
                websiteTextView.setText("");

                if (places.getStatus().isSuccess()) {
                    Place myPlace = places.get(0);

                    infoTextView.setText(
                            myPlace.getName().toString() + "\n" +
//                                    myPlace.getWebsiteUri() + "\n" +
                                    myPlace.getPhoneNumber() + "\n" +
                                    getStars(myPlace.getRating()) + "\n" +
                                    getDollarSigns(myPlace.getPriceLevel()));

                    if (myPlace.getWebsiteUri() != null) {
                        websiteTextView.setText(myPlace.getWebsiteUri().toString());
                    }
                }

                address = places.get(0).getAddress().toString();
                setMarker(latlng, locationName, address);
                Log.d("MainActivity", places.get(0).getName().toString());
                places.release();
            }
        });

    }

    public String getStars(float n) {
        String STAR = "★";
        String stars = " ";
        for(int i = 0; i < Math.round(n); i++) {
            stars = stars + STAR;
        }
        return stars;
    };

    public String getDollarSigns(int n) {
        String dollars = " ";
        for(int i = 0; i < n; i++) {
            dollars = dollars + "$";
        }
        return dollars;
    }


    private class MyTask extends AsyncTask<String, String, MeetUp> {

        @Override
        protected MeetUp doInBackground(String... params) {
            MeetUp meetUp = MeetUpAPI.getUpcomingMeetup();
            return meetUp;
        }

        @Override
        protected void onPostExecute(MeetUp meetUp) {
            locationName = meetUp.getLocName();
            latlng= new LatLng(meetUp.getLat(), meetUp.getLng());
            address = meetUp.getAddress();
            meetUpDesc = meetUp.getDescription();
            gotoLocation(latlng.latitude, latlng.longitude, zoom);
            addInfo(IMapConstants.HAYWARD_TECHIES + "\n" +
                    "Next Meeting: " + meetUp.getNextMeetingDate() +
                    "Time: " + meetUp.getNextMeetingTime(), meetUp.getWebsite());
            setMarker(latlng, locationName, address);
        }
    }


}